@@ -244,24 +244,39 @@ constrain_pairwise <- function(constraints, coefs, reg_ex = FALSE, with_zero = F
 #' Several different small-sample corrections are available.
 #'
 #' @param obj Fitted model for which to calculate Wald tests.
-#' @param constraints List of one or more constraints to test. See details and
-#'   examples.
+#' @param constraints constraint or list of multiple constraints to test. See
+#'   details and examples.
 #' @param vcov Variance covariance matrix estimated using \code{vcovCR} or a
 #'   character string specifying which small-sample adjustment should be used to
 #'   calculate the variance-covariance.
+#' @param null_constant vector of null values or list of such vectors for each
+#'   set of constraints to test. For a single \code{constraint}, the null values
+#'   must have length equal to the number of rows in the constraint. For lists
+#'   of null values, each entry must have length equal to the number of rows in
+#'   the corresponding entry of \code{constraints}. Default is \code{0}, in
+#'   which case the null values are taken to be zero (for every entry, if
+#'   \code{constraints} is a list).
 #' @param test Character vector specifying which small-sample correction(s) to
 #'   calculate. The following corrections are available: \code{"chi-sq"},
-#'   \code{"Naive-F"}, \code{"Naive-Fp"}, \code{"HTA"}, \code{"HTB"}, \code{"HTZ"}, \code{"EDF"},
-#'   \code{"EDT"}. Default is \code{"HTZ"}.
+#'   \code{"Naive-F"}, \code{"Naive-Fp"}, \code{"HTA"}, \code{"HTB"},
+#'   \code{"HTZ"}, \code{"EDF"}, \code{"EDT"}. Default is \code{"HTZ"}.
 #' @param tidy Logical value controlling whether to tidy the test results. If
-#'   \code{constraints} is a list with multiple constraints, the result will
-#'   be coerced into a data frame when \code{tidy = TRUE}.
+#'   \code{constraints} is a list with multiple constraints, the result will be
+#'   coerced into a data frame when \code{tidy = TRUE}.
+#' @param adjustment_method A character string indicating a multiple comparisons
+#'   correction to apply to p-values in instances where multiple tests are run.
+#'   Possible options are from \code{\link[stats]{p.adjust.methods}}, which is
+#'   passed to \code{\link[stats]{p.adjust}} to correct p-values for multiple
+#'   comparisons. Defaults to \code{"none"}.
 #' @param ... Further arguments passed to \code{\link{vcovCR}}, which are only
 #'   needed if \code{vcov} is a character string.
 #'
 #' @details Constraints can be specified directly as q X p matrices or
 #'   indirectly through \code{\link{constrain_equal}},
-#'   \code{\link{constrain_zero}}, or \code{\link{constrain_pairwise}}
+#'   \code{\link{constrain_zero}}, or \code{\link{constrain_pairwise}}. By
+#'   default, each constraint will be tested against the null hypothesis that it
+#'   equal to a zero vector. Non-zero values for null-hypotheses can be
+#'   specified using the \code{null_constant} argument.
 #'
 #' @return A list of test results.
 #'
@@ -272,7 +287,7 @@ constrain_pairwise <- function(constraints, coefs, reg_ex = FALSE, with_zero = F
 #'
 #'
 #' if (requireNamespace("carData", quietly = TRUE)) withAutoprint({
-#' 
+#'
 #' data(Duncan, package = "carData")
 #' Duncan$cluster <- sample(LETTERS[1:8], size = nrow(Duncan), replace = TRUE)
 #'
@@ -304,13 +319,31 @@ constrain_pairwise <- function(constraints, coefs, reg_ex = FALSE, with_zero = F
 #' Wald_test(Duncan_fit,
 #'           constraints = constrain_pairwise(":income", reg_ex = TRUE, with_zero = TRUE),
 #'           vcov = "CR2", cluster = Duncan$cluster)
-#'           
+#'
+#' # Pairwise comparisons of type-by-education slopes, with two tests and 
+#' # multiple comparisons p-value adjustment
+#' Wald_test(Duncan_fit,
+#'           constraints = constrain_pairwise(":education", reg_ex = TRUE),
+#'           vcov = "CR2",
+#'           cluster = Duncan$cluster,
+#'           test = c("HTZ","chi-sq"),
+#'           adjustment_method = "holm")
+#'
 #' })
 #'
 #' @export
 
 
-Wald_test <- function(obj, constraints, vcov, test = "HTZ", tidy = FALSE, ...) {
+Wald_test <- function(
+    obj, 
+    constraints, 
+    vcov, 
+    null_constant = 0,
+    test = "HTZ", 
+    tidy = FALSE, 
+    adjustment_method = "none",
+    ...
+) {
   
   if (is.character(vcov)) vcov <- vcovCR(obj, type = vcov, ...)
   if (!inherits(vcov, "clubSandwich")) stop("Variance-covariance matrix must be a clubSandwich.")
@@ -340,7 +373,41 @@ Wald_test <- function(obj, constraints, vcov, test = "HTZ", tidy = FALSE, ...) {
       stop(paste0("Constraints must be a q X ", p," matrix, a list of such matrices, or a call to a constrain_*() function."))
     }
     
-    results <- lapply(constraints, Wald_testing, beta = beta, vcov = vcov, test = test, p = p, GH = GH, stop_on_NPD = FALSE)
+    q_constraints <- sapply(constraints, nrow)
+    
+    null_consts_error_txt <- "Each null_constant must be a numeric vector with length equal to the number of rows in the corresponding constraint matrix."
+    
+    if (is.list(null_constant)) {
+      if (length(null_constant) != length(constraints)) stop("null_constant must be a single vector or a list with the same number of entries as the constraint list.")
+      check_numeric <- sapply(null_constant, is.numeric)
+      if (!all(check_numeric)) stop(null_consts_error_txt)
+    } else if (is.numeric(null_constant)) {
+      if (length(null_constant) > 1L) {
+        null_constant <- rep(list(null_constant), length(constraints))
+      } else {
+        null_constant <- lapply(q_constraints, function(x) rep(null_constant, x))
+      }
+    } else {
+      stop(null_consts_error_txt)
+    }
+    
+    q_null <- lengths(null_constant)
+    mismatches <- q_constraints != q_null
+    if (any(mismatches)) {
+      which_mis <- which(mismatches)
+      mismatch_msg <- c(
+        null_consts_error_txt,
+        paste0("Constraint ", which_mis, " has ", q_constraints[mismatches], " rows; null constant ", which_mis, " is length ", q_null[mismatches],".")
+      )
+      stop(paste(mismatch_msg, collapse = " "))
+    }
+    
+    results <- mapply(
+      Wald_testing, 
+      C_mat = constraints, null_constant = null_constant, 
+      MoreArgs = list(beta = beta, vcov = vcov, test = test, p = p, GH = GH, stop_on_NPD = FALSE),
+      SIMPLIFY = FALSE
+    )
     
     if (tidy) {
       results <- mapply(
@@ -351,17 +418,58 @@ Wald_test <- function(obj, constraints, vcov, test = "HTZ", tidy = FALSE, ...) {
       class(results) <- c("Wald_test_clubSandwich",class(results))
     }
     
+    # p-value adjustment
+    
+    adjustment_method <- match.arg(adjustment_method, p.adjust.methods, several.ok = FALSE)
+    
+    if (adjustment_method != "none" & length(results) == 1) {
+      warning("Only one p-value is available. No p-value adjustment will be performed.")
+    } else if (adjustment_method != "none") { # skip if adjustment_method == "none"
+      if (tidy) {
+        results$p_val <- with(results, ave(p_val, test, FUN = function(p) p.adjust(p, method = adjustment_method)))
+      } else {
+        # Extract p-values
+        p_values <- lapply(results, function(x) x$p_val)
+        p_values <- matrix(unlist(p_values), ncol = length(results))
+        
+        # Perform p-value adjustment
+        p_values <- apply(p_values, MARGIN = 1, p.adjust, adjustment_method, simplify = TRUE)
+        
+        # Apply adjusted p-values to results
+        for (i in seq_along(results)) {
+          results[[i]]$p_val <- p_values[i,]
+        }
+      }
+    }
+    
   } else {
     
     
     if (!inherits(constraints, "matrix") | ncol(constraints) != p) {
       stop(paste0("Constraints must be a q X ", p," matrix, a list of such matrices, or a call to a constrain_*() function."))
     }
+  
+    null_consts_error_txt <- "null_constant must be a numeric vector with length equal to the number of rows in the constraint matrix."
     
-    results <- Wald_testing(C_mat = constraints, beta = beta, vcov = vcov, test = test, p = p, GH = GH) 
+    if (is.numeric(null_constant)) {
+      if (length(null_constant) == 1L) {
+        null_constant <- rep(null_constant, nrow(constraints))
+      } else {
+        if (length(null_constant) != nrow(constraints)) {
+          stop(null_consts_error_txt)
+        }
+      }
+    } else {
+      stop(null_consts_error_txt)
+    }
+    
+    results <- Wald_testing(
+      C_mat = constraints, null_constant = null_constant, 
+      beta = beta, vcov = vcov, test = test, p = p, GH = GH
+    ) 
   }
   
-  results
+  return(results)
 
 }
 
@@ -371,7 +479,7 @@ array_multiply <- function(mat, arr) {
   array(new_mat, dim = c(nrow(mat), dim(arr)[2], dim(arr)[3]))
 }
 
-Wald_testing <- function(C_mat, beta, vcov, test, p, GH, stop_on_NPD = TRUE) {
+Wald_testing <- function(C_mat, null_constant, beta, vcov, test, p, GH, stop_on_NPD = TRUE) {
   
   q <- nrow(C_mat)
   dims <- dim(GH$H)
@@ -411,7 +519,7 @@ Wald_testing <- function(C_mat, beta, vcov, test, p, GH, stop_on_NPD = TRUE) {
       )
     }
   } else {
-    C_beta <- C_mat %*% beta
+    C_beta <- (C_mat %*% beta - matrix(null_constant, ncol = 1L))
     Q <- as.numeric(t(C_beta) %*% inverse_vcov %*% C_beta)
     
     result <- data.frame()
